package com.bizmatics.controller.web;


import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.model.DeviceAnalogVariableList;
import com.bizmatics.model.vo.DeviceAnalogVariableListOneVo;
import com.bizmatics.model.vo.DeviceAnalogVariableListVo;
import com.bizmatics.service.DeviceAnalogVariableListService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 设备管理-变量列表
 *
 * @author ya
 * @since 2021-09-24
 */
@RestController
@RequestMapping("/deviceAnalogVariableList")
public class DeviceAnalogVariableListController {

    @Autowired
    private DeviceAnalogVariableListService deviceAnalogVariableListService;

    /**
     * 站点管理-变量列表-新增
     *
     * @param deviceAnalogVariableList
     * @return
     */
    @PostMapping("variableListAdd")
    public ApiResult<Void> variableListAdd(@RequestBody DeviceAnalogVariableList deviceAnalogVariableList) {
        deviceAnalogVariableListService.variableListAdd(deviceAnalogVariableList);
        return ApiResult.success();
    }

    /**
     * 站点管理-变量列表-修改
     *
     * @param deviceAnalogVariableList
     * @return
     */
    @PostMapping("variableListUpdate")
    public ApiResult<Void> variableListUpdate(@RequestBody DeviceAnalogVariableList deviceAnalogVariableList) {
        deviceAnalogVariableListService.variableListUpdate(deviceAnalogVariableList);
        return ApiResult.success();
    }

    /**
     * 站点管理-变量列表-注销
     *
     * @param id 变量列表主键ID
     * @return
     */

    @GetMapping("variableListDel")
    public ApiResult<Void> variableListDel(@RequestParam int id) {
        deviceAnalogVariableListService.variableListDel(id);
        return ApiResult.success();
    }

//    /**
//     * 站点管理-变量列表-列表查询-禁用
//     *
//     * @param deviceCode 设备编号
//     * @param screen     筛选条件
//     * @param size       页数
//     * @param current    条数
//     * @return
//     */
//
//    @GetMapping("variableList")
//    public ApiResult<CommonPage<DeviceAnalogVariableList>> variableList(@RequestParam(required = false) String deviceCode,
//                                                                        @RequestParam(required = false) String screen,
//                                                                        @RequestParam(value = "size", required = false, defaultValue = "1") int size,
//                                                                        @RequestParam(value = "current", required = false, defaultValue = "10") int current) {
//        return ApiResult.success(deviceAnalogVariableListService.variableList(deviceCode, screen, size, current));
//    }


    /**
     * 站点管理-变量列表-列表查询
     *
     * @param deviceAnalogVariableListVo
     * @return
     */
    @PostMapping("variableListone")
    public ApiResult<CommonPage<DeviceAnalogVariableListOneVo>> variableListone(@RequestBody DeviceAnalogVariableListVo deviceAnalogVariableListVo) {
        return ApiResult.success(deviceAnalogVariableListService.variableListone(deviceAnalogVariableListVo));
    }

//    /**
//     * 监控设备-变量列表-新增
//     *
//     * @param deviceAnalogVariableListId 变量列表ID 英文逗号隔开
//     * @param monitoringEquipmentId      监控设备ID
//     * @return
//     */
//    @GetMapping("variableListAddOne")
//    public ApiResult<Void> variableListAddOne(@RequestParam String deviceAnalogVariableListId,
//                                              @RequestParam Integer monitoringEquipmentId) {
//        deviceAnalogVariableListService.variableListAddOne(deviceAnalogVariableListId, monitoringEquipmentId);
//        return ApiResult.success();
//    }

    /**
     * 站点管理-变量列表-（监控设备为零）变量下拉框
     *
     * @return
     */
    @GetMapping("variableListDroplist")
    public ApiResult<List<DeviceAnalogVariableList>> variableListDroplist() {

        return ApiResult.success(deviceAnalogVariableListService.variableListDroplist());
    }


    /**
     * 变量列表-导出
     * @param siteId 站点ID
     * @param variableName 变量名称
     * @param dataArea 1.模拟量 2.状态量 3.参数量
     * @return
     */
    @GetMapping("variableListExport")
    public ApiResult<String> variableListExport(@RequestParam Integer siteId,
                                                @RequestParam(required = false) String variableName,
                                                @RequestParam(value = "dataArea", required = false, defaultValue = "1") Integer dataArea) {
        return ApiResult.success(deviceAnalogVariableListService.variableListExport(siteId,variableName,dataArea));
    }

    /**
     * 变量列表-导入
     * @param file 导入文件
     * @return
     * @throws Exception
     */
    @PostMapping("/variableListImport")
    public ApiResult<Void> variableListImport(@RequestParam("file") MultipartFile file){
        deviceAnalogVariableListService.variableListImport(file);
        return ApiResult.success();
    }

}

