package com.bizmatics.controller.web;


import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.model.DeviceAttribute;
import com.bizmatics.model.vo.MonitorDeviceListVO;
import com.bizmatics.service.DeviceAttributeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 设备管理-监控设备
 *
 * @author ya
 * @since 2021-09-24
 */
@RestController
@RequestMapping("/deviceAttribute")
public class DeviceAttributeController {
    @Autowired
    private DeviceAttributeService deviceAttributeService;

    /**
     * 站点管理/设备管理-监控设备-新增
     *
     * @param deviceAttribute
     * @return
     */
    @PostMapping("deviceNewsAdd")
    public ApiResult<Void> deviceNewsAdd(@RequestBody DeviceAttribute deviceAttribute) {
        deviceAttributeService.deviceNewsAdd(deviceAttribute);
        return ApiResult.success();
    }


    /**
     * 站点管理/设备管理-监控设备-修改
     *
     * @param deviceAttribute
     * @return
     */
    @PostMapping("deviceNewsUpdate")
    public ApiResult<Void> deviceNewsUpdate(@RequestBody DeviceAttribute deviceAttribute) {
        deviceAttributeService.deviceNewsUpdate(deviceAttribute);
        return ApiResult.success();
    }

    /**
     * 站点管理/设备管理-监控设备-设备列表查询
     *
     * @param id      device_attribute表主键ID
     * @param siteId  点位ID
     * @param size    条数
     * @param current 页数
     * @return
     */
    @GetMapping("deviceNewsList")
    public ApiResult<CommonPage<DeviceAttribute>> deviceNewsList(@RequestParam Integer siteId,
                                                                 @RequestParam(value = "id", required = false, defaultValue = "0") Integer id,
                                                                 @RequestParam(value = "size", required = false, defaultValue = "15") Integer size,
                                                                 @RequestParam(value = "current", required = false, defaultValue = "1") Integer current
    ) {
        return ApiResult.success(deviceAttributeService.deviceNewsList(id, siteId, size, current));
    }

    /**
     * 监控设备下拉框
     * @param siteId 点位ID
     * @param id 监控设备ID
     * @return
     */
    @GetMapping("deviceNewsListBox")
    public ApiResult<List<DeviceAttribute>> deviceNewsListBox(@RequestParam(value = "siteId", required = false, defaultValue = "0") Integer siteId,
                                                              @RequestParam(value = "id", required = false, defaultValue = "0") Integer id
    ) {
        return ApiResult.success(deviceAttributeService.deviceNewsListBox(id, siteId));
    }


    /**
     * 站点管理/设备管理-监控设备-注销
     *
     * @param id
     * @return
     */
    @GetMapping("deviceNewsDel")
    public ApiResult<Void> deviceNewsDel(@RequestParam Integer id) {
        deviceAttributeService.deviceNewsDel(id);
        return ApiResult.success();
    }

    /**
     * 站点管理/设备管理-监控设备-主页列表查询
     *
     * @param siteName 站点名称
     * @return
     */
    @GetMapping("monitorDeviceList")
    public ApiResult<CommonPage<MonitorDeviceListVO>> monitorDeviceList(@RequestParam(required = false) String siteName,
                                                                        @RequestParam(value = "size", required = false, defaultValue = "15") Integer size,
                                                                        @RequestParam(value = "current", required = false, defaultValue = "1") Integer current,
                                                                        @RequestParam(value = "userName", required = false) String userName,
                                                                        @RequestParam(value = "phone", required = false) String phone) {
        return ApiResult.success(deviceAttributeService.monitorDeviceList(siteName,userName,phone, size, current));
    }

    /**
     * 站点管理/设备管理-监控设备-列表导出
     *
     * @param siteId 站点ID
     * @param id     监控漫设备ID
     * @return
     */
    @GetMapping("deviceAttributeExport")
    public ApiResult<String> deviceAttributeExport(@RequestParam Integer siteId,
                                                   @RequestParam(value = "id", required = false, defaultValue = "0") Integer id
    ) {
        return ApiResult.success(deviceAttributeService.deviceAttributeExport(id, siteId));
    }

    /**
     * 监控设备-导入
     * @param file 导入文件
     * @return
     * @throws Exception
     */
    @PostMapping("/deviceAttributeImport")
    public ApiResult<Void> deviceAttributeImport(@RequestParam("file") MultipartFile file) {
        deviceAttributeService.deviceAttributeImport(file);
        return ApiResult.success();
    }

}

