package com.bizmatics.controller.web;


import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.model.PatrolInspectionDevice;
import com.bizmatics.model.vo.PatrolInspectionDeviceVo;
import com.bizmatics.service.PatrolInspectionDeviceService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * 巡检管理-巡检设备
 *
 * @author ya
 * @since 2021-10-15
 */
@RestController
@RequestMapping("/patrolInspectionDevice")
public class PatrolInspectionDeviceController {

    @Autowired
    private PatrolInspectionDeviceService patrolInspectionDeviceService;

    /**
     * 巡检管理-巡检设备-新增
     *
     * @param inspectionDeviceName 巡检设备名称
     * @param siteId               站点ID
     * @param deviceTypeId         设备类型
     * @return
     */
    @GetMapping("patrolInspectionDeviceAdd")
    public ApiResult<Void> patrolInspectionDeviceAdd(@RequestParam String inspectionDeviceName,
                                                     @RequestParam Integer siteId,
                                                     @RequestParam Integer deviceTypeId
    ) {
        patrolInspectionDeviceService.patrolInspectionDeviceAdd(inspectionDeviceName, siteId, deviceTypeId);
        return ApiResult.success();
    }

    /**
     * 巡检管理-巡检设备-修改
     *
     * @param patrolInspectionDevice
     * @return
     */
    @PostMapping("patrolInspectionDeviceUpdate")
    public ApiResult<Void> patrolInspectionDeviceUpdate(@RequestBody PatrolInspectionDevice patrolInspectionDevice
    ) {
        patrolInspectionDeviceService.patrolInspectionDeviceUpdate(patrolInspectionDevice);
        return ApiResult.success();
    }

    /**
     * 巡检管理-巡检设备-注销
     *
     * @param id 巡检设备ID
     * @return
     */
    @GetMapping("patrolInspectionDeviceDel")
    public ApiResult<Void> patrolInspectionDeviceDel(@RequestParam Integer id
    ) {
        patrolInspectionDeviceService.patrolInspectionDeviceDel(id);
        return ApiResult.success();
    }

    /**
     * 巡检管理-巡检设备-列表与回显查询
     *
     * @param id                   巡检设备ID
     * @param siteId               站点ID
     * @param inspectionDeviceName 巡检设备名称
     * @return
     */
    @GetMapping("patrolInspectionDeviceList")
    public ApiResult<List<PatrolInspectionDeviceVo>> patrolInspectionDeviceList(@RequestParam(value = "id", required = false, defaultValue = "0") Integer id,
                                                                                @RequestParam(value = "siteId", required = false, defaultValue = "0") Integer siteId,
                                                                                @RequestParam(required = false) String inspectionDeviceName
    ) {
        return ApiResult.success(patrolInspectionDeviceService.patrolInspectionDeviceList(inspectionDeviceName, siteId, id));
    }


    /**
     * 通用图片上传
     *
     * @param file    图片
     * @param request
     * @return
     */
    @PostMapping("pictureUpload")
    public ApiResult<String> pictureUpload(@RequestParam(value = "file", required = false) MultipartFile file, HttpServletRequest request
    ) {
        return ApiResult.success(patrolInspectionDeviceService.pictureUpload(file, request));
    }


}

