package com.bizmatics.controller.web;


import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.model.vo.CurrencyDroplistVo;
import com.bizmatics.model.vo.PatrolInspectionRecordDetailsVo;
import com.bizmatics.model.vo.PatrolInspectionRecordVo;
import com.bizmatics.service.PatrolInspectionRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 巡检管理-巡检记录
 *
 * @author ya
 * @since 2021-10-20
 */
@RestController
@RequestMapping("/patrolInspectionRecord")
public class PatrolInspectionRecordController {


    @Autowired
    private PatrolInspectionRecordService patrolInspectionRecordService;

    /**
     * 巡检管理-巡检记录-按计划查看/按站点查看
     *
     * @param siteId             站点ID
     * @param inspectionSchemeId 巡检计划ID
     * @param size               页数
     * @param current            条数
     * @return
     */
    @GetMapping("patrolInspectionRecordList")
    public ApiResult<CommonPage<PatrolInspectionRecordVo>> patrolInspectionRecordList(@RequestParam(value = "siteId", required = false, defaultValue = "0") Integer siteId,
                                                                                      @RequestParam(value = "inspectionSchemeId", required = false, defaultValue = "0") Integer inspectionSchemeId,
                                                                                      @RequestParam(value = "size", required = false, defaultValue = "1") Integer size,
                                                                                      @RequestParam(value = "current", required = false, defaultValue = "15") Integer current
    ) {
        return ApiResult.success(patrolInspectionRecordService.patrolInspectionRecordList(inspectionSchemeId, siteId, size, current));
    }

    /**
     * 巡检管理-巡检记录-详情查询
     *
     * @param id 记录ID
     * @return
     */
    @GetMapping("patrolInspectionRecordDetails")
    public ApiResult<List<PatrolInspectionRecordDetailsVo>> patrolInspectionRecordDetails(@RequestParam Integer id
    ) {
        return ApiResult.success(patrolInspectionRecordService.patrolInspectionRecordDetails(id));
    }

    /**
     * 巡检管理-巡检记录-按站点查看下拉框
     *
     * @return
     */
    @GetMapping("patrolInspectionRecordSiteDroplist")
    public ApiResult<List<CurrencyDroplistVo>> patrolInspectionRecordSiteDroplist(
    ) {
        return ApiResult.success(patrolInspectionRecordService.patrolInspectionRecordSiteDroplist());
    }

    /**
     * 巡检管理-巡检记录-按计划查看下拉框
     *
     * @return
     */
    @GetMapping("patrolInspectionRecordPlanDroplist")
    public ApiResult<List<CurrencyDroplistVo>> patrolInspectionRecordPlanDroplist(
    ) {
        return ApiResult.success(patrolInspectionRecordService.patrolInspectionRecordPlanDroplist());
    }
}

