package com.bizmatics.controller.web;


import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.model.PlatformArea;
import com.bizmatics.model.vo.DeviceVo;
import com.bizmatics.model.vo.PlatformAreaVo;
import com.bizmatics.model.vo.SiteDeviceList;
import com.bizmatics.model.vo.SiteListVo;
import com.bizmatics.service.PlatformAreaService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * 台区管理
 *
 * @author ya
 * @since 2021-09-29
 */
@RestController
@RequestMapping("/platformArea")
public class PlatformAreaController {

    @Autowired
    private PlatformAreaService platformAreaService;

    /**
     * 台区管理-新增
     *
     * @param platformArea
     * @return
     */
    @PostMapping("platformAreaAdd")
    public ApiResult<Void> platformAreaAdd(@RequestBody PlatformArea platformArea) {
        platformAreaService.platformAreaAdd(platformArea);
        return ApiResult.success();
    }

    /**
     * 台区管理-修改
     *
     * @param platformArea
     * @return
     */
    @PostMapping("platformAreaUpdate")
    public ApiResult<Void> platformAreaUpdate(@RequestBody PlatformArea platformArea) {
        platformAreaService.platformAreaUpdate(platformArea);
        return ApiResult.success();
    }


    /**
     * 台区管理-修改回显
     *
     * @param id
     * @return
     */
    @GetMapping("platformAreaList")
    public ApiResult<List<PlatformAreaVo>> platformAreaList(@RequestParam(value = "id", required = false, defaultValue = "0") Integer id) {
        return ApiResult.success(platformAreaService.platformAreaList(id));
    }

    /**
     * 台区管理-列表查询
     *
     * @param platformAreaName 台区名称
     * @param size 条数
     * @param current 页数
     * @return
     */
    @GetMapping("platformAreaListOne")
    public ApiResult<CommonPage<PlatformAreaVo>> platformAreaListOne(@RequestParam(required = false) String platformAreaName,
                                                                     @RequestParam(value = "size", required = false, defaultValue = "15") Integer size,
                                                                     @RequestParam(value = "current", required = false, defaultValue = "1") Integer current) {
        return ApiResult.success(platformAreaService.platformAreaListOne(platformAreaName,current,size));
    }


    /**
     * 台区管路-注销
     *
     * @param id 台区ID
     * @return
     */
    @GetMapping("platformAreaDel")
    public ApiResult<Void> platformAreaDel(@RequestParam int id) {
        platformAreaService.platformAreaDel(id);
        return ApiResult.success();
    }


    /**
     * 台区管路-站点设备列表
     *
     * @param stationAreaId
     * @return
     */
    @GetMapping("siteDeviceList")
    public ApiResult<List<SiteListVo>> siteDeviceList(@RequestParam Integer stationAreaId) {
        return ApiResult.success(platformAreaService.siteDeviceList(stationAreaId));
    }

    /**
     * 台区管理-站点设备修改
     *
     * @param deviceVo
     * @return
     */
    @PostMapping("siteDeviceUpdate")
    public ApiResult<Void> siteDeviceUpdate(@RequestBody DeviceVo deviceVo) {
        platformAreaService.siteDeviceUpdate(deviceVo);
        return ApiResult.success();
    }


    /**
     * 台区管理-站点设备注销
     *
     * @param deviceId
     * @param deviceAttributeId
     * @return
     */
    @GetMapping("siteDeviceDel")
    public ApiResult<Void> siteDeviceDel(@RequestParam int deviceId,
                                         @RequestParam int deviceAttributeId) {
        platformAreaService.siteDeviceDel(deviceId, deviceAttributeId);
        return ApiResult.success();
    }
}

