package com.bizmatics.controller.web;


import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.service.RtAnalogDataService;
import com.bizmatics.service.vo.RadCountVO;
import com.bizmatics.service.vo.RealScoreOneVO;
import com.bizmatics.service.vo.RealScoreVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 实时消费组模拟量
 *
 * @author ya
 * @since 2021-07-07
 */
@RestController
@RequestMapping("/rtAnalogData")
public class RtAnalogDataController {


    @Autowired
    private RtAnalogDataService rtAnalogDataService;

    /**
     * 重载/轻载/正常
     *
     * @return
     */
    @GetMapping("count")
    public ApiResult<RadCountVO> getCount() {
        return ApiResult.success(rtAnalogDataService.selectCount());
    }

    /**
     * 查看实时数据详情
     *
     * @param siteId 站点id
     * @return
     */
    @GetMapping("one")
    public ApiResult<List<Map<String, Object>>> getOne(@RequestParam Integer siteId) {
        return ApiResult.success(rtAnalogDataService.getOne(siteId));
    }


    /**
     * 实时负荷
     *
     * @param siteId 站点id
     * @return
     */
    @GetMapping("epLoad")
    public ApiResult<Double> getEpLoad(Integer siteId) {
        return ApiResult.success(rtAnalogDataService.getEpLoad(siteId));
    }

    /**
     * 电能质量(实时评分)
     *
     * @param deviceCode 设备编号
     * @return
     */
    @GetMapping("realScore")
    public ApiResult<RealScoreVO> realScore(@RequestParam String deviceCode) {
        return ApiResult.success(rtAnalogDataService.realScore(deviceCode));
    }

    /**
     * 电能质量(单回路监测)
     *
     * @param deviceCode 设备编号
     * @return
     */
    @GetMapping("realScoreOne")
    public ApiResult<RealScoreOneVO> realScoreOne(@RequestParam String deviceCode) {
        return ApiResult.success(rtAnalogDataService.realScoreOne(deviceCode));
    }

    /**
     * 伍继PC1.0-数据报表
     *
     * @param siteId 站点ID
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param queryType 类型 日 月 年
     * @return
     */
    @RequestMapping("DataReport")
    public ApiResult<List<Map<String, Object>>> getDataReport(@RequestParam Integer siteId,
                                                              @RequestParam(required = false) Date startTime,
                                                              @RequestParam(required = false) Date endTime,
                                                              @RequestParam(required = true) String queryType) {
        return ApiResult.success(rtAnalogDataService.getDataReport(siteId, startTime, endTime, queryType));
    }

    /**
     * 伍继PC1.0-历史趋势数据查询
     * @param siteId 站点ID
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param queryType 类型 日 月 年
     * @return
     */
    @RequestMapping("HistoricalCurve")
    public ApiResult<List<Map<String, Object>>> getHistoricalCurve(@RequestParam Integer siteId,
                                                                   @RequestParam(required = false) Date startTime,
                                                                   @RequestParam(required = false) Date endTime,
                                                                   @RequestParam(required = true) String queryType) {
        return ApiResult.success(rtAnalogDataService.getHistoricalCurve(siteId, startTime, endTime, queryType));
    }

    /**
     * 伍继PC1.0-数据报表导出
     * @param siteId 站点ID
     * @param startTime 开始时间
     * @param endTime 结束时间
     * @param queryType 类型 日 月 年
     * @return
     */
    @GetMapping("/DataReportExport")
    public ApiResult<String> export(@RequestParam Integer siteId,
                                    @RequestParam(required = false) Date startTime,
                                    @RequestParam(required = false) Date endTime,
                                    @RequestParam(required = true) String queryType) {
        return ApiResult.success(rtAnalogDataService.DataReportExport(siteId, startTime, endTime, queryType));
    }

}

