package com.bizmatics.controller.web;


import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.model.DeviceAnalogVariableList;
import com.bizmatics.model.Route;
import com.bizmatics.model.Site;
import com.bizmatics.model.TemplateData;
import com.bizmatics.service.SiteService;
import com.bizmatics.service.vo.DeviceCountVO;
import com.bizmatics.service.vo.SiteVO;
import com.bizmatics.service.vo.SiteVOT;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * 站点
 *
 * @author ya
 * @since 2021-07-07
 */
@RestController
@RequestMapping("/site")
public class SiteController {


    @Autowired
    private SiteService siteService;

    /**
     * 查看不同状态的站点数量
     *
     * @return
     */
    @GetMapping("count")
    public ApiResult<DeviceCountVO> getCount() {
        return ApiResult.success(siteService.selectCount());
    }


    /**
     * 查看站点列表
     *
     * @param name 名称
     * @return
     */
    @GetMapping("list")
    public ApiResult<List<Site>> list(@RequestParam(required = false) String name) {
        return ApiResult.success(siteService.list(name));
    }

    /**
     * 未知
     * @param name 站点名称
     * @return
     */

    @GetMapping("listStatus")
    public ApiResult<List<SiteVO>> siteAndStatus(@RequestParam(required = false) String name) {
        return ApiResult.success(siteService.siteAndStatusList(name));
    }

    /**
     * 查看站点详情
     *
     * @param siteId 站点id
     * @return
     */
    @GetMapping("one")
    public ApiResult<SiteVO> getOne(@RequestParam Long siteId) {
        return ApiResult.success(siteService.getOne(siteId));
    }

    /**
     * 查看站点列表
     *
     * @param name 站点名称
     * @return
     */
    @GetMapping("siteList")
    public ApiResult<List<Site>> siteList(@RequestParam(required = false) String name) {
        return ApiResult.success(siteService.siteList(name));
    }


    /**
     *站点管理-基础信息-新增修改
     * @param siteVOT
     * @return
     */
    @PostMapping("siteAdd")
    public ApiResult<Void> siteAdd(@RequestBody SiteVOT siteVOT) {
        if (siteVOT.getId()==0){
            siteService.siteAdd(siteVOT);
        }else {
            siteService.siteUpdate(siteVOT);
        }
        return ApiResult.success();
    }

    /**
     * 站点管理-基础信息-站点注销
     * @param id
     * @return
     */
    @GetMapping("siteDel")
    public ApiResult<Void> siteDel(@RequestParam(required = false) String id) {
            siteService.sitedel(id);
        return ApiResult.success();
    }

    /**
     * 站点管理-基础信息-详情查询
     * @param id 站点ID
     * @return
     */
    @GetMapping("getSite")
    public ApiResult<List<TemplateData>> getSite(@RequestParam String id) {
        return ApiResult.success(siteService.getSite(id));
    }

    /**
     * 站点管理-右侧树形目录
     * @param name
     * @return
     */
    @GetMapping("sitelist")
    public ApiResult<List<Map<String,Object>>> sitelist(@RequestParam(required = false) String name){
        return ApiResult.success(siteService.sitelist(name));
    }

    /**
     * 站点管理-基础信息-动态属性下拉框
     * @param siteId 站点管理
     * @param dataArea 1.模拟量 2.状态量 3.参数量
     * @return
     */
    @GetMapping("dynamicPropertiesDroplist")
    public ApiResult<List<DeviceAnalogVariableList>> dynamicPropertiesDroplist(@RequestParam(value = "siteId", required = false, defaultValue = "0") Integer siteId,
                                                                               @RequestParam(value = "dataArea", required = false, defaultValue = "1") Integer dataArea) {
        return ApiResult.success(siteService.dynamicPropertiesDroplist(siteId, dataArea));
    }


    /**
     * 站点管理-基础信息-线路下拉框
     * @return
     */
    @GetMapping("routeDroplist")
    public ApiResult<List<Route>> dynamicPropertiesDroplist() {
        return ApiResult.success(siteService.routeDroplist());
    }
}

