package com.bizmatics.service.config.security.handle;

import com.bizmatics.common.core.bean.ApiResult;
import com.bizmatics.common.mvc.utils.ServletUtils;
import com.bizmatics.common.spring.util.JsonUtils;
import com.bizmatics.model.constants.Constants;
import com.bizmatics.service.system.impl.TokenService;
import com.bizmatics.service.config.security.LoginUser;
import com.bizmatics.service.util.manager.AsyncManager;
import com.bizmatics.service.util.manager.factory.AsyncFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.core.Authentication;
import org.springframework.security.web.authentication.logout.LogoutSuccessHandler;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Objects;


/**
 * 自定义退出处理类 返回成功
 * 
 * @author yq
 */
@Configuration
public class LogoutSuccessHandlerImpl implements LogoutSuccessHandler
{
    @Autowired
    private TokenService tokenService;

    /**
     * 退出处理
     * 
     * @return
     */
    @Override
    public void onLogoutSuccess(HttpServletRequest request, HttpServletResponse response, Authentication authentication)
            throws IOException, ServletException
    {
        LoginUser loginUser = tokenService.getLoginUser(request);
        if (Objects.nonNull(loginUser))
        {
            String userName = loginUser.getUsername();
            // 删除用户缓存记录
            tokenService.delLoginUser(loginUser.getToken());
            // 记录用户退出日志
            AsyncManager.me().execute(AsyncFactory.recordLogininfor(userName, Constants.LOGOUT, "退出成功"));
        }
        ServletUtils.renderString(response, JsonUtils.toJson(ApiResult.success("退出成功")));
    }
}
