package com.bizmatics.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.ExcelImportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.ImportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.common.core.exception.BusinessException;
import com.bizmatics.common.core.util.BeanMapperUtils;
import com.bizmatics.common.core.util.FileUtils;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.common.spring.util.GlobalUtils;
import com.bizmatics.model.AlarmPower;
import com.bizmatics.model.DeviceAnalogVariableList;
import com.bizmatics.model.DeviceAttribute;
import com.bizmatics.model.system.SysUser;
import com.bizmatics.model.vo.DeviceAnalogVariableListOneVo;
import com.bizmatics.model.vo.DeviceAnalogVariableListVo;
import com.bizmatics.persistence.mapper.DeviceAnalogVariableListMapper;
import com.bizmatics.service.DeviceAnalogVariableListService;
import com.bizmatics.service.util.SecurityUtils;
import com.bizmatics.service.vo.*;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 设备变量列表 服务实现类
 *
 * @author ya
 * @since 2021-09-24
 */
@Service
public class DeviceAnalogVariableListServiceImpl extends AbstractCrudService<DeviceAnalogVariableListMapper, DeviceAnalogVariableList> implements DeviceAnalogVariableListService {

    @Override
    public void variableListAdd(DeviceAnalogVariableList deviceAnalogVariableList) {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        deviceAnalogVariableList.setVariableCoding(deviceAnalogVariableList.getDeviceCode()+"_"+deviceAnalogVariableList.getVariableCoding());
        deviceAnalogVariableList.setCreateTime(new Date());
        deviceAnalogVariableList.setCreator(user.getUserName());
        deviceAnalogVariableList.setStatus(1);
        this.save(deviceAnalogVariableList);
    }

    @Override
    public void variableListUpdate(DeviceAnalogVariableList deviceAnalogVariableList) {
        this.updateById(deviceAnalogVariableList);
    }

    @Override
    public void variableListDel(int id) {
        DeviceAnalogVariableList deviceAnalogVariableList = new DeviceAnalogVariableList();
        deviceAnalogVariableList.setId(id);
        deviceAnalogVariableList.setStatus(0);
        this.updateById(deviceAnalogVariableList);
    }

    @Override
    public CommonPage<DeviceAnalogVariableList> variableList(String deviceCode, String screen, int size, int current) {
        IPage<DeviceAnalogVariableList> page = new Page<DeviceAnalogVariableList>(size, current);
        LambdaQueryWrapper<DeviceAnalogVariableList> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DeviceAnalogVariableList::getDeviceCode, deviceCode).eq(DeviceAnalogVariableList::getStatus, 1);
        if (screen != null && !screen.equals("")) {
            queryWrapper.and(wq -> wq
                    .like(DeviceAnalogVariableList::getVariableName, screen)
                    .or()
                    .like(DeviceAnalogVariableList::getVariableCoding, screen));
        }

        page = this.page(page, queryWrapper);
        this.ToCommonPage(page);
        return new CommonPage<>(page.getRecords(), page.getTotal(), page.getCurrent(), page.getSize());
    }

    @Override
    public CommonPage<DeviceAnalogVariableListOneVo> variableListone(DeviceAnalogVariableListVo deviceAnalogVariableListVo) {
        int start_current = (deviceAnalogVariableListVo.getCurrent() - 1) * deviceAnalogVariableListVo.getSize();
        List<DeviceAnalogVariableListOneVo> deviceAnalogVariableListOne = baseMapper.variableList(deviceAnalogVariableListVo.getSiteId(), deviceAnalogVariableListVo.getId(),
                deviceAnalogVariableListVo.getVariableName(), null, null, deviceAnalogVariableListVo.getDataArea(),deviceAnalogVariableListVo.getMonitoringEquipment());
        int total = 0;
        if (deviceAnalogVariableListOne.size() > 0) {
            total = deviceAnalogVariableListOne.size();
        }
        List<DeviceAnalogVariableListOneVo> deviceAnalogVariableList = baseMapper.variableList(deviceAnalogVariableListVo.getSiteId(), deviceAnalogVariableListVo.getId(),
                deviceAnalogVariableListVo.getVariableName(), start_current, deviceAnalogVariableListVo.getSize(), deviceAnalogVariableListVo.getDataArea(),deviceAnalogVariableListVo.getMonitoringEquipment());
        return new CommonPage<>(deviceAnalogVariableList, total, deviceAnalogVariableListVo.getSize(), deviceAnalogVariableListVo.getCurrent());
    }

    @Override
    public void variableListAddOne(String deviceAnalogVariableListId,Integer monitoringEquipmentId) {
        DeviceAnalogVariableList deviceAnalogVariableList = new DeviceAnalogVariableList();
        String[] idarr = deviceAnalogVariableListId.split("");

        for (int i = 0; i < idarr.length; i++) {
            deviceAnalogVariableList.setId(Integer.parseInt(idarr[i]));
            deviceAnalogVariableList.setMonitoringEquipment(monitoringEquipmentId);
            this.updateById(deviceAnalogVariableList);
        }
    }

    @Override
    public List<DeviceAnalogVariableList> variableListDroplist() {
        LambdaQueryWrapper<DeviceAnalogVariableList> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DeviceAnalogVariableList::getMonitoringEquipment, 0).eq(DeviceAnalogVariableList::getStatus, 1);
        List<DeviceAnalogVariableList> DeviceAnalogVariableListDroplist = this.list(queryWrapper);
        return DeviceAnalogVariableListDroplist;
    }

    @Override
    public String variableListExport( Integer siteId,String variableName,Integer dataArea) {
        Workbook workbook = null;
        File file = null;
        try {
            ExportParams params = new ExportParams(null, "变量列表");
            workbook = ExcelExportUtil.exportBigExcel(params, DeviceAnalogVariableListExportVO.class,
                    (o, i) -> {
                        Page<DeviceAnalogVariableListOneVo> page = new Page<>(i, 30);
                        page = baseMapper.variableListExport(page,siteId, variableName, dataArea);
                        return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), DeviceAnalogVariableListOneVo.class, DeviceAnalogVariableListExportVO.class));
                    }, null);
            if (null != workbook) {
                file = FileUtils.getFile(GlobalUtils.getTempBaseDir(), String.format("%s-%s.xlsx", "变量列表", System.currentTimeMillis() + ""));
                FileUtils.createFile(file.getAbsolutePath());
                FileOutputStream allListingFileOutputStream = new FileOutputStream(file);
                workbook.write(allListingFileOutputStream);
            } else {
                throw new BusinessException("表格数据为空");
            }
        } catch (Exception e) {
            log.error("导出文件失败", e);
            throw new BusinessException("导出文件失败");
        } finally {
            if (workbook != null) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    log.error("===export spec=== 关闭workbook失败", e);
                }
            }
        }
        return file.getName();
    }

    @Transactional
    @Override
    public void variableListImport(MultipartFile multipartFile){
        SysUser user = SecurityUtils.getLoginUser().getUser();
        ImportParams params = new ImportParams();
        params.setHeadRows(1);
        String err="文件导入失败";
        try {
            List<DeviceAnalogVariableListImport> deviceAnalogVariableListImportList = ExcelImportUtil.importExcel(multipartFile.getInputStream(),
                    DeviceAnalogVariableListImport.class, params);
            if (CollectionUtils.isNotEmpty(deviceAnalogVariableListImportList)){
                int rot=0;
                for (DeviceAnalogVariableListImport deviceAnalogVariableListImportLists:deviceAnalogVariableListImportList) {
                    DeviceAnalogVariableList deviceAnalogVariableList = BeanMapperUtils.map(deviceAnalogVariableListImportLists, DeviceAnalogVariableList.class);
                    deviceAnalogVariableList.setVariableCoding(deviceAnalogVariableListImportLists.getDeviceCode()+"_"+deviceAnalogVariableListImportLists.getVariableCoding());
                    deviceAnalogVariableList.setStatus(1);
                    deviceAnalogVariableList.setCreator(user.getUserName());
                    deviceAnalogVariableList.setCreateTime(new Date());
                    try{
                        this.save(deviceAnalogVariableList);
                    }catch (Exception e){
                        int h=rot+2;
                        err="文件导入失败，第"+h+"行数据导入失败";
                        throw  new BusinessException(err);
                    }
                    rot++;
                }
            }else {
                err="文件不能为空";
                throw new BusinessException(err);
            }
        }catch (Exception e){
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            throw  new BusinessException(err);
        }
    }


}
