package com.bizmatics.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.ExcelImportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.ImportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.common.core.exception.BusinessException;
import com.bizmatics.common.core.util.BeanMapperUtils;
import com.bizmatics.common.core.util.FileUtils;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.common.spring.util.GlobalUtils;
import com.bizmatics.model.Device;
import com.bizmatics.model.DeviceAttribute;
import com.bizmatics.model.system.SysUser;
import com.bizmatics.model.vo.MonitorDeviceListVO;
import com.bizmatics.persistence.mapper.DeviceAttributeMapper;
import com.bizmatics.service.DeviceAttributeService;
import com.bizmatics.service.DeviceService;
import com.bizmatics.service.util.SecurityUtils;
import com.bizmatics.service.vo.DeviceAttributeExportVO;
import com.bizmatics.service.vo.DeviceAttributeImportVo;
import com.bizmatics.service.vo.DeviceImportVo;
import org.apache.poi.ss.usermodel.Workbook;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 设备属性 服务实现类
 *
 * @author ya
 * @since 2021-09-24
 */
@SuppressWarnings("unused")
@Service
public class DeviceAttributeServiceImpl extends AbstractCrudService<DeviceAttributeMapper, DeviceAttribute> implements DeviceAttributeService {

    @Autowired
    private DeviceService deviceService;

    @Override
    public void deviceNewsAdd(DeviceAttribute deviceAttribute) {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        deviceAttribute.setStatus(1);
        deviceAttribute.setCreator(user.getUserName());
        deviceAttribute.setCreateTime(new Date());
        this.save(deviceAttribute);
    }

    @Override
    public void deviceNewsUpdate(DeviceAttribute deviceAttribute) {
        this.updateById(deviceAttribute);
    }

    @Override
    public  CommonPage<DeviceAttribute> deviceNewsList(Integer id, Integer siteId,Integer size,Integer current) {
        IPage<DeviceAttribute> page = new Page<DeviceAttribute>(current, size);
        LambdaQueryWrapper<DeviceAttribute> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DeviceAttribute::getStatus, 1).eq(DeviceAttribute::getSiteId, siteId);
        if (id != 0) {
            queryWrapper.eq(DeviceAttribute::getId, id);
        }
        page = this.page(page, queryWrapper);
        this.ToCommonPage(page);
        return new CommonPage<>(page.getRecords(), page.getTotal(), page.getSize(),page.getCurrent());
    }

    @Override
    public  List<DeviceAttribute> deviceNewsListBox(Integer id, Integer siteId){
        LambdaQueryWrapper<DeviceAttribute> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DeviceAttribute::getStatus, 1);
        if (siteId!=0){
            queryWrapper.eq(DeviceAttribute::getSiteId, siteId);
        }
        if (id != 0) {
            queryWrapper.eq(DeviceAttribute::getId, id);
        }
        return this.list(queryWrapper);
    }

    @Override
    public void deviceNewsDel(int id) {
        DeviceAttribute deviceAttribute = new DeviceAttribute();
        deviceAttribute.setId(id);
        deviceAttribute.setStatus(0);
        this.updateById(deviceAttribute);
    }

    @Override
    public CommonPage<MonitorDeviceListVO> monitorDeviceList(String siteName ,String userName,String phone,Integer size,Integer current) {
        List<MonitorDeviceListVO> monitorDeviceListOne = baseMapper.monitorDeviceList(siteName,userName,phone,null,0);
        Integer total = 0;
        if (monitorDeviceListOne.size()>0){
            total = monitorDeviceListOne.size();
        }
        Integer startCurrent = (current - 1) * size;
        List<MonitorDeviceListVO> monitorDeviceList = baseMapper.monitorDeviceList(siteName,userName,phone,startCurrent,size);
        return new CommonPage<>(monitorDeviceList, total, size,current);
    }


    @Override
    public String deviceAttributeExport(Integer id, Integer siteId) {
        Integer userId = SecurityUtils.getLoginUser().getUser().getUserId().intValue();
        Workbook workbook = null;
        File file = null;
        try {
            ExportParams params = new ExportParams(null, "监控设备列表");
            workbook = ExcelExportUtil.exportBigExcel(params, DeviceAttributeExportVO.class,
                    (o, i) -> {
                        Page<DeviceAttribute> page = new Page<DeviceAttribute>(i, 30);
                        LambdaQueryWrapper<DeviceAttribute> queryWrapper = Wrappers.lambdaQuery();
                        queryWrapper.eq(DeviceAttribute::getStatus, 1).eq(DeviceAttribute::getSiteId, siteId);
                        if (id != 0) {
                            queryWrapper.eq(DeviceAttribute::getId, id);
                        }
                        page = this.page(page, queryWrapper);
//                        this.ToCommonPage(page);
                        return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), DeviceAttribute.class, DeviceAttributeExportVO.class));
                    }, null);
            if (null != workbook) {
                file = FileUtils.getFile(GlobalUtils.getTempBaseDir(), String.format("%s-%s.xlsx", "监控设备列表", System.currentTimeMillis() + ""));
                FileUtils.createFile(file.getAbsolutePath());
                FileOutputStream allListingFileOutputStream = new FileOutputStream(file);
                workbook.write(allListingFileOutputStream);
            } else {
                throw new BusinessException("表格数据为空");
            }
        } catch (Exception e) {
            log.error("导出文件失败", e);
            throw new BusinessException("导出文件失败");
        } finally {
            if (workbook != null) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    log.error("===export spec=== 关闭workbook失败", e);
                }
            }
        }
        return file.getName();
    }


    @Transactional
    @Override
    public void deviceAttributeImport(MultipartFile multipartFile){
        SysUser user = SecurityUtils.getLoginUser().getUser();
        ImportParams params = new ImportParams();
        params.setHeadRows(1);
        String err="文件导入失败";
        try {
            List<DeviceAttributeImportVo> deviceAttributeImportVoList = ExcelImportUtil.importExcel(multipartFile.getInputStream(),
                    DeviceAttributeImportVo.class, params);
            if (CollectionUtils.isNotEmpty(deviceAttributeImportVoList)){
                int rot=0;
                for (DeviceAttributeImportVo deviceAttributeImportVos:deviceAttributeImportVoList) {
                    DeviceAttribute deviceAttribute = BeanMapperUtils.map(deviceAttributeImportVos, DeviceAttribute.class);
                    deviceAttribute.setVariableListId(0);
                    deviceAttribute.setStatus(1);
                    deviceAttribute.setCreator(user.getUserName());
                    deviceAttribute.setCreateTime(new Date());
                    try{
                        this.save(deviceAttribute);
                    }catch (Exception e){
                        int h=rot+2;
                        err="文件导入失败，第"+h+"行数据导入失败";
                        throw  new BusinessException(err);
                    }
                    rot++;
                }
            }else {
                err="文件不能为空";
                throw new BusinessException(err);
            }
        }catch (Exception e){
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            throw  new BusinessException(err);
        }
    }
}
