package com.bizmatics.service.impl;


import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.bizmatics.common.core.exception.BusinessException;
import com.bizmatics.common.core.util.Arith;
import com.bizmatics.common.core.util.BeanMapperUtils;
import com.bizmatics.common.core.util.DateUtils;
import com.bizmatics.common.core.util.FileUtils;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.common.spring.util.GlobalUtils;
import com.bizmatics.common.spring.util.JsonUtils;
import com.bizmatics.model.*;
import com.bizmatics.model.vo.DataManagementOneVO;
import com.bizmatics.model.vo.DataManagementVO;
import com.bizmatics.model.vo.HtAnalogDataOneVo;
import com.bizmatics.model.vo.HtAnalogDataVo;
import com.bizmatics.persistence.mapper.HtAnalogDataMapper;
import com.bizmatics.service.*;
import com.bizmatics.service.util.FieldEscapeUtils;
import com.bizmatics.service.util.SecurityUtils;
import com.bizmatics.service.vo.*;
import com.fasterxml.jackson.core.type.TypeReference;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;


/**
 * @author yq
 * @date 2021/7/20 16:49
 */
@Service
public class HtAnalogDataServiceImpl extends AbstractCrudService<HtAnalogDataMapper, HtAnalogData> implements HtAnalogDataService {


    @Autowired
    private HadSiteStaticService hadSiteStaticService;
    @Autowired
    private RtAnalogDataService rtAnalogDataService;

    @Autowired
    private DeviceService deviceService;
    @Autowired
    private SiteDynamicPropertiesService siteDynamicPropertiesService;
    @Autowired
    private DeviceAttributeService deviceAttributeService;

    @Override
    public HadCountVO selectCount() {
        HadCountVO hadCountVO = new HadCountVO();
        Integer userId = SecurityUtils.getLoginUser().getUser().getUserId().intValue();
        Date date = new Date();
        hadCountVO.setDayCount(hadSiteStaticService.getCount(userId, DateUtils.getDayStartTime(date), date, null));
        hadCountVO.setMonthCount(hadSiteStaticService.getCount(userId, DateUtils.getFirstDayOfMonth(date), date, null));
        hadCountVO.setYearCount(hadSiteStaticService.getCount(userId, DateUtils.getBeginDayOfYear(date), date, null));
        return hadCountVO;
    }

    @Override
    public List<CommonIcoVO> selectTrendByDate(Integer siteId) {
        Integer userId = SecurityUtils.getLoginUser().getUser().getUserId().intValue();
        Date date = new Date();
        Date sTime = DateUtils.getDayStartTime(date);
        Date eTime = DateUtils.getDayEndTime(date);
        Date ysTime = DateUtils.addDays(sTime, -1);
        Date yeTime = DateUtils.addDays(eTime, -1);
        List<CommonIcoVO> list = new ArrayList<>();
        if (null != siteId) {
            list.add(getTreedBySite("昨日", siteId, ysTime, yeTime));
            list.add(getTreedBySite("今日", siteId, sTime, eTime));
        } else {
            list.add(getTrendOne("昨日", userId, ysTime, yeTime));
            list.add(getTrendOne("今日", userId, sTime, eTime));
        }
        return list;
    }


    public CommonIcoVO getTrendOne(String name, Integer userId, Date startTime, Date endTime) {
        List<HadSiteStatic> list = hadSiteStaticService.list(null, startTime, endTime, userId);
        List<Object> objects = new ArrayList<>();
        objects.add(0.0);
        List<Object> dates = new ArrayList<>();
        dates.add("00:00:00");
        for (int i = 2; i < 24; i += 2) {
            //结束时间
            Date hours = DateUtils.setHours(startTime, i);
            startTime = hours;
            dates.add(DateUtils.getTime(hours));
            Double hourCount = 0.00;
            for (HadSiteStatic hadSiteStatic : list) {
                CommonIcoVO commonIcoVO = JsonUtils.fromJson(hadSiteStatic.getLoadIco(), CommonIcoVO.class);
                List<Double> dataList = (List<Double>) commonIcoVO.getList();
                hourCount += dataList.get(i / 2);
            }
            objects.add(Arith.div(hourCount, 1, 2));
        }
        return CommonIcoVO.builder().name(name).list(objects).listDate(dates).build();
    }

    CommonIcoVO getTreedBySite(String name, Integer siteId, Date startTime, Date endTime) {
        Date sTime = DateUtils.getDayStartTime(startTime);
        Date eTime = DateUtils.getDayEndTime(endTime);
        HadSiteStatic hadSiteStatic = hadSiteStaticService.oneBySite(siteId, sTime, eTime);
        CommonIcoVO commonIcoVO = null;
        if (null != hadSiteStatic) {
            commonIcoVO = JsonUtils.fromJson(hadSiteStatic.getLoadIco(), CommonIcoVO.class);
            commonIcoVO.setName(name);
        } else {
            List<Object> objects = new ArrayList<>();
            objects.add(0.0);
            List<Object> dates = new ArrayList<>();
            dates.add("00:00:00");
            for (int i = 2; i < 24; i += 2) {
                //结束时间
                Date hours = DateUtils.setHours(startTime, i);
                objects.add(0.0);
                startTime = hours;
                dates.add(DateUtils.getTime(hours));
            }
            commonIcoVO = CommonIcoVO.builder().name(name).list(objects).listDate(dates).build();
        }
        return commonIcoVO;
    }

    @Override
    public List<CommonIcoVO> selectTrendByMonth() {
        Date date = new Date();
        Integer userId = SecurityUtils.getLoginUser().getUser().getUserId().intValue();
        List<CommonIcoVO> list = new ArrayList<>();
        list.add(selectTreedMonthOne(DateUtils.getLastDayOfMonth(DateUtils.addMonths(date, -1)), userId, "上月趋势图"));
        list.add(selectTreedMonthOne(DateUtils.getLastDayOfMonth(date), userId, "当月趋势图"));
        return list;
    }

    public CommonIcoVO selectTreedMonthOne(Date date, Integer userId, String name) {
        List<Object> objects = new ArrayList<>();
        List<Object> dateList = new ArrayList<>();
        for (int i = 1; i <= Integer.parseInt(DateUtils.getDay(date)); i++) {
            Date setDays = DateUtils.setDays(date, i);
            Date dayStartTime = DateUtils.getDayStartTime(setDays);
            Date dayEndTime = DateUtils.getDayEndTime(setDays);
            objects.add(hadSiteStaticService.getCount(userId, dayStartTime, dayEndTime, null));
            dateList.add(DateUtils.getDay(setDays));
        }
        return CommonIcoVO.builder().name(name).list(objects).listDate(dateList).build();
    }

    @Override
    public HadCountVO getCountBySite(Integer siteId) {
        Integer userId = SecurityUtils.getLoginUser().getUser().getUserId().intValue();
        Date date = new Date();
        //当日开始时间
        Date firstDayOfDate = DateUtils.getDayStartTime(date);
        //当月开始时间
        Date firstDayOfMonth = DateUtils.getFirstDayOfMonth(date);
        //当年开始时间
        Date firstDayOfYear = DateUtils.getBeginDayOfYear(date);
        HadCountVO hadCountVO = new HadCountVO();
        hadCountVO.setDayCount(hadSiteStaticService.getCount(userId, DateUtils.getDayStartTime(date), date, siteId));
        hadCountVO.setMonthCount(hadSiteStaticService.getCount(userId, DateUtils.getFirstDayOfMonth(date), date, siteId));
        hadCountVO.setYearCount(hadSiteStaticService.getCount(userId, DateUtils.getBeginDayOfYear(date), date, siteId));
        hadCountVO.setLastDayCount(hadSiteStaticService.getCount(userId, DateUtils.addDays(firstDayOfDate, -1), DateUtils.addDays(date, -1), siteId));
        hadCountVO.setLastMonthCount(hadSiteStaticService.getCount(userId, DateUtils.addMonths(firstDayOfMonth, -1), DateUtils.addMonths(date, -1), siteId));
        hadCountVO.setLastYearCount(hadSiteStaticService.getCount(userId, DateUtils.addYears(firstDayOfYear, -1), DateUtils.addYears(date, -1), siteId));
        hadCountVO.setDayRadio(Optional.ofNullable(hadCountVO.getDayCount())
                .filter(dayCount -> 0.00 != dayCount)
                .map(dayCount -> Arith.div(hadCountVO.getLastDayCount(), dayCount))
                .orElse(0.00));
        hadCountVO.setMonthRadio(Optional.ofNullable(hadCountVO.getMonthCount())
                .filter(dayCount -> 0.00 != dayCount)
                .map(dayCount -> Arith.div(hadCountVO.getLastMonthCount(), dayCount))
                .orElse(0.00));
        hadCountVO.setYearRadio(Optional.ofNullable(hadCountVO.getYearCount())
                .filter(dayCount -> 0.00 != dayCount)
                .map(dayCount -> Arith.div(hadCountVO.getLastYearCount(), dayCount))
                .orElse(0.00));
        return hadCountVO;

    }


    @Override
    public TimeShareVO getTimeShare(Integer siteId, Integer type) {
        TimeShareVO timeShareVO = new TimeShareVO();
        Date date = new Date();
        if (0 == type) {
            HadSiteStatic hadSiteStatic = hadSiteStaticService.oneBySite(siteId, date, date);
            Optional.ofNullable(hadSiteStatic).ifPresent(hads -> JsonUtils.fromJson(hads.getTimeShare(), timeShareVO.getClass()));
        } else {
            List<HadSiteStatic> list = hadSiteStaticService.list(siteId, DateUtils.getFirstDayOfMonth(date), date);
            for (HadSiteStatic hadSiteStatic : list) {
                Optional.ofNullable(hadSiteStatic.getTimeShare())
                        .ifPresent(has -> {
                            TimeShareVO ts = JsonUtils.fromJson(has, TimeShareVO.class);
                            timeShareVO.setFlat(Arith.add(timeShareVO.getFlat(), ts.getFlat()));
                            timeShareVO.setPeak(Arith.add(timeShareVO.getPeak(), ts.getPeak()));
                            timeShareVO.setNeed(Arith.add(timeShareVO.getNeed(), ts.getNeed()));
                            timeShareVO.setGrain(Arith.add(timeShareVO.getGrain(), ts.getGrain()));
                        });
            }
        }
        return timeShareVO;
    }

    @Override
    public List<CommonIcoVO> getDemandIco(Integer siteId) {
        Date date = new Date();
        List<Object> maxList = new ArrayList<>();
        List<Object> minList = new ArrayList<>();
        List<Object> avgList = new ArrayList<>();
        List<Object> dateList = new ArrayList<>();
        List<HadSiteStatic> hadSiteStatics = hadSiteStaticService.list(siteId, DateUtils.getFirstDayOfMonth(date), date);
        for (int i = 1; i <= Integer.parseInt(DateUtils.getDay(date)); i++) {
            Date setDays = DateUtils.setDays(date, i);
            Date dayStartTime = DateUtils.getDayStartTime(setDays);
            Date dayEndTime = DateUtils.getDayEndTime(setDays);
            maxList.add(hadSiteStatics.stream()
                    .filter(hadSiteStatic -> DateUtils.isEffectiveDate(hadSiteStatic.getHadTime(), dayStartTime, dayEndTime)
                    ).findFirst().map(HadSiteStatic::getMaxDemand).orElse(0.00));
            minList.add(hadSiteStatics.stream()
                    .filter(hadSiteStatic -> DateUtils.isEffectiveDate(hadSiteStatic.getHadTime(), dayStartTime, dayEndTime)
                    ).findFirst().map(HadSiteStatic::getMinDemand).orElse(0.00));
            avgList.add(hadSiteStatics.stream()
                    .filter(hadSiteStatic -> DateUtils.isEffectiveDate(hadSiteStatic.getHadTime(), dayStartTime, dayEndTime)
                    ).findFirst().map(HadSiteStatic::getAvgDemand).orElse(0.00));
            dateList.add(DateUtils.getDay(setDays));
        }
        List<CommonIcoVO> list = new ArrayList<>();
        list.add(CommonIcoVO.builder().name("最大值").list(maxList).listDate(dateList).build());
        list.add(CommonIcoVO.builder().name("最小值").list(minList).listDate(dateList).build());
        list.add(CommonIcoVO.builder().name("平均值").list(avgList).listDate(dateList).build());
        return list;
    }

    @Override
    public List<CommonIcoVO> getElectricIco(Integer siteId, Date date) {
        List<Object> iaList = new ArrayList<>();
        iaList.add(0.00);
        List<Object> ibList = new ArrayList<>();
        ibList.add(0.00);
        List<Object> icList = new ArrayList<>();
        icList.add(0.00);
        List<Object> uaList = new ArrayList<>();
        uaList.add(0.00);
        List<Object> ubList = new ArrayList<>();
        ubList.add(0.00);
        List<Object> ucList = new ArrayList<>();
        ucList.add(0.00);
        List<Object> dateList = new ArrayList<>();
        dateList.add("00:00:00");
        HadSiteStatic hadSiteStatic = hadSiteStaticService.oneBySite(siteId, date, date);
        List<HtAnalogData> htList = Optional.ofNullable(hadSiteStatic)
                .map(HadSiteStatic::getHisIco)
                .map(his -> JsonUtils.fromJson(hadSiteStatic.getHisIco(), new TypeReference<List<HtAnalogData>>() {
                }))
                .orElse(null);
        for (int i = 2; i < 24; i += 2) {
            Date hours = DateUtils.setHours(date, i);
            date = hours;
            Date finalDate = date;
            Optional<HtAnalogData> htAnalogData = Optional.ofNullable(htList).flatMap(hts -> hts.stream()
                    .filter(hads -> DateUtils.isEffectiveDate(hads.getDataTime(), finalDate, hours))
                    .findFirst());
            iaList.add(htAnalogData.map(HtAnalogData::getIa).orElse(0.00));
            ibList.add(htAnalogData.map(HtAnalogData::getIb).orElse(0.00));
            icList.add(htAnalogData.map(HtAnalogData::getIc).orElse(0.00));
            uaList.add(htAnalogData.map(HtAnalogData::getUa).orElse(0.00));
            ubList.add(htAnalogData.map(HtAnalogData::getUb).orElse(0.00));
            ucList.add(htAnalogData.map(HtAnalogData::getUc).orElse(0.00));
            dateList.add(htAnalogData.map(HtAnalogData::getDataTime).map(s -> DateUtils.getTime(hours)).orElse(DateUtils.getTime(hours)));
        }
        List<CommonIcoVO> list = new ArrayList<>();
        list.add(CommonIcoVO.builder().name("IA").list(iaList).listDate(dateList).build());
        list.add(CommonIcoVO.builder().name("IB").list(ibList).listDate(dateList).build());
        list.add(CommonIcoVO.builder().name("IC").list(icList).listDate(dateList).build());
        list.add(CommonIcoVO.builder().name("UA").list(uaList).listDate(dateList).build());
        list.add(CommonIcoVO.builder().name("UB").list(ubList).listDate(dateList).build());
        list.add(CommonIcoVO.builder().name("UC").list(ucList).listDate(dateList).build());
        return list;
    }

    @Override
    public List<RealScoreVO> rtRealScore(String deviceCode, Date startTime, Date endTime) {
        LambdaQueryWrapper<HtAnalogData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(HtAnalogData::getIa, HtAnalogData::getIb, HtAnalogData::getIc,
                HtAnalogData::getUa, HtAnalogData::getUb, HtAnalogData::getUc, HtAnalogData::getDataTime,
                HtAnalogData::getCos)
                .eq(HtAnalogData::getDeviceName, deviceCode)
                .between(HtAnalogData::getDataTime, startTime, endTime);
        List<HtAnalogData> list = this.list(queryWrapper);
        //查询设备
        LambdaQueryWrapper<Device> deviceQuery = Wrappers.lambdaQuery();
        deviceQuery.eq(Device::getDeviceCode, deviceCode);
        Device device = deviceService.getOne(deviceQuery);
        Optional.ofNullable(device).orElseThrow(() -> new BusinessException("设备不存在"));
        //查询sd
        LambdaQueryWrapper<SiteDynamicProperties> sdQuery = Wrappers.lambdaQuery();
        sdQuery.eq(SiteDynamicProperties::getSiteId, device.getSiteId());
        SiteDynamicProperties siteDynamicProperties = siteDynamicPropertiesService.getOne(sdQuery);
        Optional.ofNullable(siteDynamicProperties).orElseThrow(() -> new BusinessException("sd不存在"));
        LambdaQueryWrapper<DeviceAttribute> adQuery = Wrappers.lambdaQuery();
        adQuery.eq(DeviceAttribute::getSiteId, device.getSiteId());
        List<DeviceAttribute> deviceAttributes = deviceAttributeService.list(adQuery);
        if (CollectionUtils.isEmpty(deviceAttributes)) {
            throw new BusinessException("da不存在");
        }
        DeviceAttribute deviceAttribute = deviceAttributes.stream().min(Comparator.comparing(DeviceAttribute::getId)).get();
        List<RealScoreVO> realScoreVOS = new ArrayList<>();
        for (HtAnalogData htAnalogData : list) {
            RtAnalogData rtAnalogData = BeanMapperUtils.map(htAnalogData, RtAnalogData.class);
            realScoreVOS.add(rtAnalogDataService.fillRealScoreData(rtAnalogData, siteDynamicProperties, deviceAttribute));
        }
        return realScoreVOS;
    }


    @Override
    public List<CommonIcoVO> yearOnYearList(DataManagementOneVO dataManagementOneVO) {
        List<HtAnalogDataVo> yearOnYearList = null;
        List<DataManagementVO> dataManagementVOList = new ArrayList<>();
        List<String> newListOne = new ArrayList<>();
        List<String> newListTwo = new ArrayList<>();
        List<String> newListThree = new ArrayList<>();
        List<DeviceAnalogVariableList> variableCodingList = new LinkedList<>();
        List<String> fieldDisplay = new ArrayList<>();
        newListOne.add("dataTime");
        for (int i = 0; i < dataManagementOneVO.getDisplayField().size(); i++) {
            DataManagementVO dataManagementVO = new DataManagementVO();
            dataManagementVO.setDisplayField(dataManagementOneVO.getDisplayField().get(i).split("_")[0]);
            dataManagementVOList.add(dataManagementVO);
            newListOne.add(dataManagementOneVO.getDisplayField().get(i).split("_")[1]);
            newListTwo.add(dataManagementOneVO.getDisplayField().get(i).split("_")[2]);
            fieldDisplay.add(dataManagementOneVO.getDisplayField().get(i).split("_")[1]);
            newListThree.add(dataManagementOneVO.getDisplayField().get(i).split("_")[1] + "_" + dataManagementOneVO.getDisplayField().get(i).split("_")[2]);
            DeviceAnalogVariableList deviceAnalogVariableList = new DeviceAnalogVariableList();
            deviceAnalogVariableList.setVariableCoding(dataManagementOneVO.getDisplayField().get(i).split("_")[0] + "_" + dataManagementOneVO.getDisplayField().get(i).split("_")[1]);
            variableCodingList.add(deviceAnalogVariableList);
        }
        Set set = new HashSet();
        List<DataManagementVO> newList = new ArrayList();
        set.addAll(dataManagementVOList);
        newList.addAll(set);

        Set set1 = new HashSet();
        List<String> list = new ArrayList();
        set1.addAll(newListOne);
        list.addAll(set1);

        Set set2 = new HashSet();
        List<String> monitor_list = new ArrayList();
        set2.addAll(newListTwo);
        monitor_list.addAll(set2);

        Set set3 = new HashSet();
        List<String> fieldDisplayOne = new ArrayList();
        set3.addAll(fieldDisplay);
        fieldDisplayOne.addAll(set3);

        if (dataManagementOneVO.getType() == 1) {
            if (dataManagementOneVO.getValueCalculation() == 1) {
                yearOnYearList = baseMapper.yearOnYearOne(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            } else if (dataManagementOneVO.getValueCalculation() == 2) {
                yearOnYearList = baseMapper.dayMaxDataList(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            } else if (dataManagementOneVO.getValueCalculation() == 3) {
                yearOnYearList = baseMapper.dayMinDataList(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            } else if (dataManagementOneVO.getValueCalculation() == 4) {
                yearOnYearList = baseMapper.daySumDataList(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            }
        } else if (dataManagementOneVO.getType() == 2) {
            if (dataManagementOneVO.getValueCalculation() == 1) {
                yearOnYearList = baseMapper.yearOnYearTwo(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            } else if (dataManagementOneVO.getValueCalculation() == 2) {
                yearOnYearList = baseMapper.monthMaxDataList(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            } else if (dataManagementOneVO.getValueCalculation() == 3) {
                yearOnYearList = baseMapper.monthMinDataList(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            } else if (dataManagementOneVO.getValueCalculation() == 4) {
                yearOnYearList = baseMapper.monthSumDataList(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            }
        } else {
            yearOnYearList = baseMapper.yearOnYearThree(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList, fieldDisplayOne);
        }
        FieldEscapeUtils.remove1(list);
        List<CommonIcoVO> reListOne = new ArrayList<>();
        if (yearOnYearList != null && !yearOnYearList.isEmpty()) {
            for (int i = 0; i < monitor_list.size(); i++) {
                for (int j = 0; j < list.size(); j++) {
                    List<Object> data = new ArrayList<>();
                    String name = null;
                    List<Object> time = new ArrayList<>();
                    for (int k = 0; k < yearOnYearList.size(); k++) {
                        String variable_one = list.get(j) + "_" + monitor_list.get(i);
                        String variable_two = list.get(j) + "_" + yearOnYearList.get(k).getDeviceAttributeId();
                        System.out.println("variable_one:" + variable_one + "---variable_two:" + variable_two);
                        if (variable_one.equals(variable_two)) {
                            String variable_three = list.get(j) + "_" + yearOnYearList.get(k).getDeviceAttributeId();
                            System.out.println("variable_one_a:" + newListThree + "---variable_two_b:" + variable_three + "---value:" + newListThree.contains(variable_three));
                            if (newListThree.contains(variable_three)) {
                                name = yearOnYearList.get(k).getMonitorDeviceName() + "/" + FieldEscapeUtils.fieldEscapeUtils(list.get(j));
                                time.add(yearOnYearList.get(k).getDataTime());
//                                System.out.println();
                                data.add(FieldEscapeUtils.transformation(yearOnYearList.get(k), list.get(j)));
                            }
                        }
                    }
                    if (data != null && !data.isEmpty()) {
                        reListOne.add(CommonIcoVO.builder().name(name).list(data).listDate(time).build());
                    }

                }
            }
        }
        return reListOne;
    }


    @Override
    public List<CommonIcoVO> ringRatioList(DataManagementOneVO dataManagementOneVO) {
        List<HtAnalogDataVo> currentPeriodRingRatioList = null;
//        List<HtAnalogDataVo> lastPeriodRingRatioList = null;
        List<DataManagementVO> dataManagementVOList = new ArrayList<>();
        List<String> newListOne = new ArrayList<>();
        List<String> newListTwo = new ArrayList<>();
        List<String> newListThree = new ArrayList<>();
        List<String> fieldDisplay = new ArrayList<>();
        newListOne.add("dataTime");
        List<DeviceAnalogVariableList> variableCodingList = new LinkedList<>();
        for (int i = 0; i < dataManagementOneVO.getDisplayField().size(); i++) {
            DataManagementVO dataManagementVO = new DataManagementVO();
            dataManagementVO.setDisplayField(dataManagementOneVO.getDisplayField().get(i).split("_")[0]);
            dataManagementVOList.add(dataManagementVO);
            newListOne.add(dataManagementOneVO.getDisplayField().get(i).split("_")[1]);
            fieldDisplay.add(dataManagementOneVO.getDisplayField().get(i).split("_")[1]);
            newListTwo.add(dataManagementOneVO.getDisplayField().get(i).split("_")[2]);
            newListThree.add(dataManagementOneVO.getDisplayField().get(i).split("_")[1] + "_" + dataManagementOneVO.getDisplayField().get(i).split("_")[2]);
            DeviceAnalogVariableList deviceAnalogVariableList = new DeviceAnalogVariableList();
            deviceAnalogVariableList.setVariableCoding(dataManagementOneVO.getDisplayField().get(i).split("_")[0] + "_" + dataManagementOneVO.getDisplayField().get(i).split("_")[1]);
            variableCodingList.add(deviceAnalogVariableList);
        }
        Set set = new HashSet();
        List<DataManagementVO> newList = new ArrayList();
        set.addAll(dataManagementVOList);
        newList.addAll(set);

        Set set1 = new HashSet();
        List<String> list = new ArrayList();
        set1.addAll(newListOne);
        list.addAll(set1);

        Set set2 = new HashSet();
        List<String> monitor_list = new ArrayList();
        set2.addAll(newListTwo);
        monitor_list.addAll(set2);

        Set set3 = new HashSet();
        List<String> fieldDisplayOne = new ArrayList();
        set3.addAll(fieldDisplay);
        fieldDisplayOne.addAll(set3);

        //旧数据时间计算
//        SimpleDateFormat sfOne = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//        long startLong = DateUtils.toDate(DateUtils.toLocalDateTime(dataManagementOneVO.getStartTime(), "yyyy-MM-dd HH:mm:ss")).getTime();
//        long endLong = DateUtils.toDate(DateUtils.toLocalDateTime(dataManagementOneVO.getEndTime(), "yyyy-MM-dd HH:mm:ss")).getTime();
//        long interval = (endLong - startLong) / 13;
//        String lastPeriodStartTime = sfOne.format(new Date(startLong - (interval * 14)));
        if (dataManagementOneVO.getType() == 1) {
            if (dataManagementOneVO.getValueCalculation() == 1) {
                currentPeriodRingRatioList = baseMapper.yearOnYearOne(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            } else if (dataManagementOneVO.getValueCalculation() == 2) {
                currentPeriodRingRatioList = baseMapper.dayMaxDataList(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            } else if (dataManagementOneVO.getValueCalculation() == 3) {
                currentPeriodRingRatioList = baseMapper.dayMinDataList(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            } else if (dataManagementOneVO.getValueCalculation() == 4) {
                currentPeriodRingRatioList = baseMapper.daySumDataList(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            }
        } else if (dataManagementOneVO.getType() == 2) {
            if (dataManagementOneVO.getValueCalculation() == 1) {
                currentPeriodRingRatioList = baseMapper.yearOnYearTwo(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            } else if (dataManagementOneVO.getValueCalculation() == 2) {
                currentPeriodRingRatioList = baseMapper.monthMaxDataList(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            } else if (dataManagementOneVO.getValueCalculation() == 3) {
                currentPeriodRingRatioList = baseMapper.monthMinDataList(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            } else if (dataManagementOneVO.getValueCalculation() == 4) {
                currentPeriodRingRatioList = baseMapper.monthSumDataList(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList);
            }
        } else {
//            String[] a={"Epn3","Epp"};
            currentPeriodRingRatioList = baseMapper.yearOnYearThree(newList, dataManagementOneVO.getStartTime(), dataManagementOneVO.getEndTime(), variableCodingList, fieldDisplayOne);
        }

        FieldEscapeUtils.remove1(list);
        List<CommonIcoVO> reListOne = new ArrayList<>();
        if (currentPeriodRingRatioList != null && !currentPeriodRingRatioList.isEmpty()) {
            for (int i = 0; i < monitor_list.size(); i++) {
                for (int j = 0; j < list.size(); j++) {
                    List<Object> data = new ArrayList<>();
                    String name = null;
                    List<Object> time = new ArrayList<>();
                    for (int k = 0; k < currentPeriodRingRatioList.size(); k++) {
                        String variable_one = list.get(j) + "_" + monitor_list.get(i);
                        String variable_two = list.get(j) + "_" + currentPeriodRingRatioList.get(k).getDeviceAttributeId();
                        System.out.println("variable_one:" + variable_one + "---variable_two:" + variable_two);
                        if (variable_one.equals(variable_two)) {
                            String variable_three = list.get(j) + "_" + currentPeriodRingRatioList.get(k).getDeviceAttributeId();
                            System.out.println("variable_one_a:" + newListThree + "---variable_two_b:" + variable_three + "---value:" + newListThree.contains(variable_three));
                            if (newListThree.contains(variable_three)) {
                                name = currentPeriodRingRatioList.get(k).getMonitorDeviceName() + "/" + FieldEscapeUtils.fieldEscapeUtils(list.get(j));
                                time.add(currentPeriodRingRatioList.get(k).getDataTime());
//                                System.out.println();
                                data.add(FieldEscapeUtils.transformation(currentPeriodRingRatioList.get(k), list.get(j)));
                            }
                        }
                    }
                    if (data != null && !data.isEmpty()) {
                        reListOne.add(CommonIcoVO.builder().name(name).list(data).listDate(time).build());
                    }

                }
            }
        }
        return reListOne;
    }

    @Override
    public List<CommonIcoVO> monthlyReport(String deviceCode, int cycle, String startTime, String endTime) {
        List<Object> eppList = new ArrayList<>();
        List<Object> freezingTimesList = new ArrayList<>();
        List<HtAnalogDataVo> maxMonthlyReport = baseMapper.energyUseList(deviceCode, startTime, endTime, cycle, 1);
        List<HtAnalogDataVo> minMonthlyReport = baseMapper.energyUseList(deviceCode, startTime, endTime, cycle, 2);
        if (maxMonthlyReport.size() > 0) {
            for (int i = 0; i < maxMonthlyReport.size(); i++) {
                BigDecimal b1 = new BigDecimal(maxMonthlyReport.get(i).getEpp().toString());
                BigDecimal b2 = new BigDecimal(minMonthlyReport.get(i).getEpp().toString());
                eppList.add(b1.subtract(b2).doubleValue());
                freezingTimesList.add(maxMonthlyReport.get(i).getFreezingTimes());
            }
        }
        List<CommonIcoVO> list = new ArrayList<>();
        list.add(CommonIcoVO.builder().name("电量").list(eppList).listDate(freezingTimesList).build());
        return list;
    }


    @Override
    public List<CommonIcoVO> demandAnalysis(String deviceCode, String monthDate, int cycle) {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat formatter1 = new SimpleDateFormat("yyyy-MM-dd");
        String date = FieldEscapeUtils.getLastDayOfMonth(monthDate);
        String date1 = monthDate + "-01 00:00:00";
        String date2 = date + " 23:59:59";
        Date startTime;
        Date endTime;
        try {
            startTime = formatter.parse(date1);
            endTime = formatter.parse(date2);
        } catch (Exception e) {
            throw new BusinessException("时间错误");
        }

        LambdaQueryWrapper<HtAnalogData> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(HtAnalogData::getDemand, HtAnalogData::getFreezingTime)
                .eq(HtAnalogData::getDeviceName, deviceCode)
                .between(HtAnalogData::getFreezingTime, startTime, endTime)
                .groupBy(HtAnalogData::getFreezingTime);
        List<HtAnalogData> list1 = this.list(queryWrapper);
        List<Object> demandList = new ArrayList<>();
        List<Object> freezingTimesList = new ArrayList<>();
        for (int i = 0; i < list1.size(); i++) {
            demandList.add(list1.get(i).getDemand());
            freezingTimesList.add(formatter1.format(list1.get(i).getFreezingTime()));
        }


//        List<Object> demandList = new ArrayList<>();
//        List<Object> freezingTimesList = new ArrayList<>();
//        List<HtAnalogDataVo> maxdemandAnalysis = baseMapper.demandAnalysisList(deviceCode, monthDate);
//        for (int i = 0; i < maxdemandAnalysis.size(); i++){
//            demandList.add(maxdemandAnalysis.get(i).getDemand());
//            freezingTimesList.add(maxdemandAnalysis.get(i).getFreezingTimes());
//        }
        List<CommonIcoVO> list = new ArrayList<>();
        list.add(CommonIcoVO.builder().name("实时有功需量").list(demandList).listDate(freezingTimesList).build());
        return list;
    }

    @Override
    public String monthlyReportExport(String deviceCode, String startTime, String endTime, int cycle) {
        Workbook workbook = null;
        File file = null;
        try {
            ExportParams params = new ExportParams(null, "用能月报");
            workbook = ExcelExportUtil.exportBigExcel(params, HtAnalogDataTwoExportVo.class,
                    (o, i) -> {
                        int startCurrent = (i - 1) * 10000;
                        List<HtAnalogDataOneVo> listOne = this.monthlyReportlist(deviceCode, startTime, endTime, cycle, startCurrent);
                        return new ArrayList<>(BeanMapperUtils.mapList(listOne, HtAnalogDataOneVo.class, HtAnalogDataTwoExportVo.class));
                    }, null);
            if (null != workbook) {
                file = FileUtils.getFile(GlobalUtils.getTempBaseDir(), String.format("%s-%s.xlsx", "用能月报", System.currentTimeMillis() + ""));
                FileUtils.createFile(file.getAbsolutePath());
                FileOutputStream allListingFileOutputStream = new FileOutputStream(file);
                workbook.write(allListingFileOutputStream);
            } else {
                throw new BusinessException("表格数据为空");
            }
        } catch (Exception e) {
            log.error("导出文件失败", e);
            throw new BusinessException("导出文件失败");
        } finally {
            if (workbook != null) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    log.error("===export spec=== 关闭workbook失败", e);
                }
            }
        }
        return file.getName();
    }


    @Override
    public List<HtAnalogDataOneVo> monthlyReportlist(String deviceCode, String startTime, String endTime, int cycle, Integer startCurrent) {
        List<HtAnalogDataOneVo> listOne = new ArrayList<>();
        List<HtAnalogDataOneVo> maxMonthlyReport = baseMapper.energyUseListOne(deviceCode, startTime, endTime, cycle, 1, 10000, startCurrent);
        List<HtAnalogDataOneVo> minMonthlyReport = baseMapper.energyUseListOne(deviceCode, startTime, endTime, cycle, 2, 10000, startCurrent);
        if (maxMonthlyReport.size() > 0) {
            Double epp=0.00;
            for (int j = 0; j < maxMonthlyReport.size()+1; j++) {
                HtAnalogDataOneVo htAnalogDataOneVo = new HtAnalogDataOneVo();
                if (j==maxMonthlyReport.size()){
                    htAnalogDataOneVo.setEpp(epp);
                    htAnalogDataOneVo.setFreezingTime("合计");
                }else {
                    BigDecimal b1 = new BigDecimal(maxMonthlyReport.get(j).getEpp().toString());
                    BigDecimal b2 = new BigDecimal(minMonthlyReport.get(j).getEpp().toString());
                    epp+=b1.subtract(b2).doubleValue();
                    htAnalogDataOneVo.setEpp(b1.subtract(b2).doubleValue());
                    htAnalogDataOneVo.setFreezingTime(maxMonthlyReport.get(j).getFreezingTime());
                }
                listOne.add(htAnalogDataOneVo);
            }
        }
        return listOne;
    }

}
