package com.bizmatics.service.impl;

import com.bizmatics.common.core.exception.BusinessException;
import com.bizmatics.common.core.util.FileUtils;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.common.spring.util.GlobalUtils;
import com.bizmatics.model.PatrolInspectionDevice;
import com.bizmatics.model.system.SysUser;
import com.bizmatics.model.vo.PatrolInspectionDeviceVo;
import com.bizmatics.persistence.mapper.PatrolInspectionDeviceMapper;
import com.bizmatics.service.PatrolInspectionDeviceService;
import com.bizmatics.service.util.SecurityUtils;
import org.jetbrains.annotations.NotNull;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.util.Date;
import java.util.List;
import java.util.UUID;

/**
 * 巡检设备
 *
 * @author ya
 * @since 2021-10-15
 */
@Service
public class PatrolInspectionDeviceServiceImpl extends AbstractCrudService<PatrolInspectionDeviceMapper, PatrolInspectionDevice> implements PatrolInspectionDeviceService {

    @Override
    public void patrolInspectionDeviceAdd(String inspectionDeviceName, Integer siteId, Integer deviceTypeId) {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        PatrolInspectionDevice patrolInspectionDevice = new PatrolInspectionDevice();
        patrolInspectionDevice.setInspectionDeviceName(inspectionDeviceName);
        patrolInspectionDevice.setSiteId(siteId);
        patrolInspectionDevice.setDeviceTypeId(deviceTypeId);
        patrolInspectionDevice.setCreator(user.getUserName());
        patrolInspectionDevice.setCreateTime(new Date());
        patrolInspectionDevice.setStatus(1);
        patrolInspectionDevice.setQrCodeAddress(UUID.randomUUID().toString());
        this.save(patrolInspectionDevice);
    }

    @Override
    public void patrolInspectionDeviceUpdate(PatrolInspectionDevice patrolInspectionDevice) {
        this.updateById(patrolInspectionDevice);
    }

    @Override
    public void patrolInspectionDeviceDel(Integer id) {
        PatrolInspectionDevice patrolInspectionDevice = new PatrolInspectionDevice();
        patrolInspectionDevice.setId(id);
        patrolInspectionDevice.setStatus(0);
        this.updateById(patrolInspectionDevice);
    }

    @Override
    public List<PatrolInspectionDeviceVo> patrolInspectionDeviceList(String inspectionDeviceName, Integer siteId, Integer id) {
        List<PatrolInspectionDeviceVo> patrolInspectionDeviceList = baseMapper.patrolInspectionDeviceList(inspectionDeviceName, siteId, id);
        return patrolInspectionDeviceList;
    }


    @Override
    public String pictureUpload(@NotNull MultipartFile file, HttpServletRequest request) {
        if (file.isEmpty()) {
            System.out.println("文件为空空");
        }
        String fileName = file.getOriginalFilename();  // 文件名
        String suffixName = fileName.substring(fileName.lastIndexOf("."));  // 后缀名
        fileName = UUID.randomUUID() + suffixName; // 新文件名

        File file1 = FileUtils.getFile(GlobalUtils.getTempBaseDir(), fileName);
        try {
            file.transferTo(file1);
        } catch (Exception e) {
            throw new BusinessException("文件上传异常", e);
        }

//        String fileName = file.getOriginalFilename();  // 文件名
//        String suffixName = fileName.substring(fileName.lastIndexOf("."));  // 后缀名
////        String filePath = "D://temp-rainy//"; // 上传后的路径
//        fileName = UUID.randomUUID() + suffixName; // 新文件名
//
//        String destFileName = request.getServletContext().getRealPath("") + "uploaded" + File.separator + fileName;
//        File dest = new File(destFileName);
////        File dest = new File(filePath + fileName);
//        if (!dest.getParentFile().exists()) {
//            dest.getParentFile().mkdirs();
//        }
//        try {
//            file.transferTo(dest);
//        } catch (IOException e) {
//            e.printStackTrace();
//        }
//        String filename = "/uploaded/" + fileName;
        return fileName;
    }

}
