package com.bizmatics.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bizmatics.common.core.bean.CommonPage;
import com.bizmatics.common.core.exception.BusinessException;
import com.bizmatics.common.core.util.BeanMapperUtils;
import com.bizmatics.common.core.util.FileUtils;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.common.spring.util.GlobalUtils;
import com.bizmatics.model.SiteArchives;
import com.bizmatics.model.Sitedtl;
import com.bizmatics.model.system.SysUser;
import com.bizmatics.model.vo.SitedtlArchivesVo;
import com.bizmatics.model.vo.SitedtlVo;
import com.bizmatics.persistence.mapper.SitedtlMapper;
import com.bizmatics.service.SiteArchivesService;
import com.bizmatics.service.SitedtlService;
import com.bizmatics.service.util.SecurityUtils;
import com.bizmatics.service.vo.SitedtlExportVO;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 运维管理-现场档案
 *
 * @author ya
 * @since 2021-10-26
 */
@Service
public class SitedtlServiceImpl extends AbstractCrudService<SitedtlMapper, Sitedtl> implements SitedtlService {

    @Autowired
    private SiteArchivesService siteArchivesService;

    @Override
    public void sitedtlAdd(SitedtlArchivesVo sitedtlArchivesVo) {
        SysUser user = SecurityUtils.getLoginUser().getUser();
        Sitedtl sitedtl = sitedtlArchivesVo.getSitedtl();
        SiteArchives siteArchives = sitedtlArchivesVo.getSiteArchives();
        sitedtl.setCreator(user.getUserName());
        sitedtl.setCreateTime(new Date());
        this.save(sitedtl);
        Integer sitedtlId = sitedtl.getId();
        siteArchives.setSitedtlId(sitedtlId);
        siteArchivesService.save(siteArchives);
    }

    public void sitedtlUpdate(SitedtlArchivesVo sitedtlArchivesVo) {
        this.updateById(sitedtlArchivesVo.getSitedtl());
        siteArchivesService.updateById(sitedtlArchivesVo.getSiteArchives());
    }

    public CommonPage<SitedtlVo> sitedtlList(Integer siteId, String electricTestOverdue, Integer size, Integer current) {
        List<SitedtlVo> sitedtlList = baseMapper.sitedtlVoList(siteId, electricTestOverdue, null, null);
        int total = 0;
        if (sitedtlList.size() > 0) {
            total = sitedtlList.size();
        }

        int startCurrent = (current - 1) * size;
        List<SitedtlVo> sitedtlListOne = baseMapper.sitedtlVoList(siteId, electricTestOverdue, startCurrent, size);
        return new CommonPage<>(sitedtlListOne, total, size, current);
    }

    @Override
    public String variableListExport(Integer siteId, String electricTestOverdue) {
        Workbook workbook = null;
        File file = null;
        try {
            ExportParams params = new ExportParams(null, "现场档案");
            workbook = ExcelExportUtil.exportBigExcel(params, SitedtlExportVO.class,
                    (o, i) -> {
                        Page<SitedtlVo> page = new Page<>(i, 30);
                        page = baseMapper.sitedtlVoListOne(page, siteId, electricTestOverdue, null, null);
                        return new ArrayList<>(BeanMapperUtils.mapList(page.getRecords(), SitedtlVo.class, SitedtlExportVO.class));
                    }, null);
            if (null != workbook) {
                file = FileUtils.getFile(GlobalUtils.getTempBaseDir(), String.format("%s-%s.xlsx", "现场档案", System.currentTimeMillis() + ""));
                FileUtils.createFile(file.getAbsolutePath());
                FileOutputStream allListingFileOutputStream = new FileOutputStream(file);
                workbook.write(allListingFileOutputStream);
            } else {
                throw new BusinessException("表格数据为空");
            }
        } catch (Exception e) {
            log.error("导出文件失败", e);
            throw new BusinessException("导出文件失败");
        } finally {
            if (workbook != null) {
                try {
                    workbook.close();
                } catch (IOException e) {
                    log.error("===export spec=== 关闭workbook失败", e);
                }
            }
        }
        return file.getName();
    }
}
