package com.bizmatics.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.bizmatics.common.core.exception.BusinessException;
import com.bizmatics.common.mvc.base.AbstractCrudService;
import com.bizmatics.model.User;
import com.bizmatics.persistence.mapper.UserMapper;
import com.bizmatics.service.UserService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Optional;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author ya
 * @since 2021-07-07
 */
@Service
public class UserServiceImpl extends AbstractCrudService<UserMapper, User> implements UserService {

    @Override
    public User getByUserName(String name) {
        return baseMapper.selectOne(Wrappers.lambdaQuery(User.class).eq(User::getUserName, name));
    }

    @Override
    public void register(User user) {
        //查询用户
        User oldUser = this.getByUserName(user.getUserName());
        if (oldUser != null) {
            throw new BusinessException("用户已存在");
        }
        //加密
        BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();
        user.setUserPassword(encoder.encode(user.getUserPassword()));
        baseMapper.insert(user);
    }
}
