package com.bizmatics.service.util;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.bizmatics.common.core.util.HttpUtils;
import com.bizmatics.common.mvc.utils.IpUtils;
import com.bizmatics.common.spring.util.JsonUtils;
import com.bizmatics.service.config.WjConfig;
import com.fasterxml.jackson.core.type.TypeReference;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;

/**
 * 获取地址类
 * 
 * @author yq
 */
public class AddressUtils
{
    private static final Logger log = LoggerFactory.getLogger(AddressUtils.class);

    // IP地址查询
    public static final String IP_URL = "http://whois.pconline.com.cn/ipJson.jsp";

    // 未知地址
    public static final String UNKNOWN = "XX XX";

    public static String getRealAddressByIP(String ip)
    {
        String address = UNKNOWN;
        // 内网不查询
        if (IpUtils.internalIp(ip))
        {
            return "内网IP";
        }
        if (WjConfig.isAddressEnabled())
        {
            try
            {
                Map<String,String> param = new HashMap<>();
                param.put("ip",ip);
                String rspStr = HttpUtils.get(IP_URL, param);
                if (StringUtils.isBlank(rspStr))
                {
                    log.error("获取地理位置异常 {}", ip);
                    return UNKNOWN;
                }
                Map<String, String> objectMap = JsonUtils.fromJson(rspStr, new TypeReference<Map<String, String>>() {
                });
                String region = objectMap.get("pro");
                String city = objectMap.get("city");
                return String.format("%s %s", region, city);
            }
            catch (Exception e)
            {
                log.error("获取地理位置异常 {}", ip);
            }
        }
        return address;
    }
}
